{% load bootstrap3 %}

<tr class="maced">
    <th class="maced" style="border: 0;"></th>
    <th class="maced" style="text-align: center; vertical-align: middle; border: 0;"> {{ item_html_name }} 1 </th>
    <th class="maced" style="text-align: center; vertical-align: middle; border: 0;"> All </th>
    <th class="maced" style="text-align: center; vertical-align: middle; border: 0; background-color: #F7D358;">
        Resulting {{ item_html_name }}
    </th>
    <th class="maced" style="text-align: center; vertical-align: middle; border: 0;"> All </th>
    <th class="maced" style="text-align: center; vertical-align: middle; border: 0;"> {{ item_html_name }} 2 </th>
</tr>
<tr class="maced">
    <th class="maced" style="border-bottom: 1px; border-top: 0;"></th>
    <th style="border-bottom: 1px; border-top: 0;">
        <select class="maced {{ item_html_class_name }} form-control" id="{{ input_id1 }}">
            {{ options_html_code|safe }}
        </select>
    </th>
    <th class="maced" style="border-bottom: 1px; border-top: 0; width: 1px;">
        <button type="button" class="maced btn btn-info" id="{{ input_id1 }}-button"
            onclick="merge_all_into_middle('{{ modal_id }}', 1);" aria-hidden="true">
            {% bootstrap_icon "arrow-right" %}
        </button>
    </th>
    <td class="maced" style="text-align: center; vertical-align: middle; background-color: #F7D358; border-bottom: 1px; border-top: 0;">
        Use buttons to copy values to the <br> middle, or type them in manually.
    </td>
    <th class="maced" style="border-bottom: 1px; border-top: 0; width: 1px;">
        <button type="button" class="maced btn btn-info" id="{{ input_id2 }}-button"
            onclick="merge_all_into_middle('{{ modal_id }}', 2);" aria-hidden="true">
            {% bootstrap_icon "arrow-left" %}
        </button>
    </th>
    <th style="border-bottom: 1px; border-top: 0;">
        <select class="maced {{ item_html_class_name }} form-control" id="{{ input_id2 }}">
            {{ options_html_code|safe }}
        </select>
    </th>
</tr>