<{% if input_type == "select" %}select{% else %}input type="{{ input_type }}"{% endif %}
    class="maced {% if input_type == "select" %}{{ item_html_class_name }} {% endif %}{{ field_html_class_name }} form-control" id="{{ input_id }}"
    {% if input_type == "color" %}
        value="#00FF00"
    {% endif %}
    {% if can_disable %}
        disabled
    {% endif %}
    {% if input_type == "select" %}>
        {{ options_html_code|safe }}
        </select
    {% endif %}
>