<tr>
    <th class="maced" style="border: 0;"> {{ field_html_name }}: </th>
    <td style="border: 0;">
        <{% if input_type == "select" %}select{% else %}input type="{{ input_type }}"{% endif %}
            class="maced {% if input_type == "select" %}{{ item_html_class_name }} {% endif %}{{ field_html_class_name }} form-control" id="{{ input_id }}"
            {% if input_type == "color" %}
                value="#00FF00"
            {% endif %}
            {% if action_type == "delete" or action_type == "clone" or action_type == "info" %}
                disabled
            {% endif %}
            {% if input_type == "select" %}>
                {{ options_html_code|safe }}
                </select
            {% endif %}
        >
    </td>
</tr>