{% include "maced/modal.html" with action_type="merge" item_html_code=merge_html_code %}
{% include "maced/modal.html" with action_type="add" item_html_code=add_html_code %}
{# {% include "maced/modal.html" with action_type="merge" item_html_code=merge_html_code %} Will be changed to clone #}
{% include "maced/modal.html" with action_type="edit" item_html_code=edit_html_code %}
{% include "maced/modal.html" with action_type="delete" item_html_code=delete_html_code %}
{# {% include "maced/modal.html" with action_type="merge" item_html_code=merge_html_code %} Will be changed to info #}